@@ -17,7 +17,7 @@ from convst.transformers._commons import (
 
 from numba import njit, prange
 
-@njit(cache=True)
+@njit(cache=True, nogil=True)
 def M_VL_init_random_shapelet_params(
     n_shapelets, shapelet_sizes, n_timestamps, p_norm, max_channels, prime_scheme
 ):
@@ -93,7 +93,7 @@ def M_VL_init_random_shapelet_params(
 
     return values, lengths, dilations, threshold, normalize, n_channels, channel_ids
 
-@njit(cache=True, parallel=True)
+@njit(cache=True, parallel=True, nogil=True)
 def M_VL_generate_shapelet(
     X, y, n_shapelets, shapelet_sizes, r_seed, p_norm, p_min, p_max, alpha,
     dist_func, use_phase, max_channels, min_len, X_len, prime_scheme
@@ -297,7 +297,7 @@ def M_VL_generate_shapelet(
         channel_ids[:a2]
     )
 
-@njit(cache=True, parallel=True, fastmath=True)
+@njit(cache=True, parallel=True, fastmath=True, nogil=True)
 def M_VL_apply_all_shapelets(
     X, shapelets, dist_func, use_phase, X_len
 ):
